/****************************************************************************
**
** Copyright (C) 2016 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the documentation of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:BSD$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** BSD License Usage
** Alternatively, you may use this file under the terms of the BSD license
** as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of The Qt Company Ltd nor the names of its
**     contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include "openglwindow.h"

#include <QGuiApplication>
#include <QMatrix4x4>
#include <QKeyEvent>
#include <QMouseEvent>
#include <QOpenGLShaderProgram>
#include <QScreen>
#include <QtMath>

#include <sstream>
#include <fstream>

//! [1]
class TriangleWindow : public OpenGLWindow
{
public:
    using OpenGLWindow::OpenGLWindow;
    
    void initialize() override;
    void render() override;
    void inicializa_matrices(void);

protected:
	void keyPressEvent( QKeyEvent * e );
	void mouseMoveEvent(QMouseEvent *e);

private:
    GLint m_posAttr = 0;
    GLint m_colAttr = 0;
    GLint m_matrixUniform = 0;

    QOpenGLShaderProgram *m_program = nullptr;
    int m_frame = 0;
	float thetax, thetay;
    // QMatrix4x4 K, RT;
};
//! [1]

//! [2]
int main(int argc, char **argv)
{
    QGuiApplication app(argc, argv);

    QSurfaceFormat format;
    format.setSamples(16);

    TriangleWindow window;
	window.inicializa_matrices();
    window.setFormat(format);
    window.resize(512, 512);
    window.setAnimating(false);
    window.show();

    return app.exec();
}

void TriangleWindow::keyPressEvent( QKeyEvent *e )
{
	 switch ( e->key() ) {
		  case Qt::Key_Escape:
		  case Qt::Key_Q:
				QCoreApplication::quit();
				break;
		  case Qt::Key_R:
				thetax = thetay = 0.0;
				break;
	 }
}


void TriangleWindow::mouseMoveEvent( QMouseEvent *e )
{
	QPoint pos = e->pos( );

	thetax = 360.0*pos.x()/600.0 - 180.0;
	// printf("%f\n", thetax );
	renderNow( );
}

void TriangleWindow::initialize()
{
    m_program = new QOpenGLShaderProgram(this);

    m_program->addShaderFromSourceFile(QOpenGLShader::Vertex, "vertexShaderSource.vertexshader" );
    m_program->addShaderFromSourceFile(QOpenGLShader::Fragment, "fragmentShaderSource.fragmentshader" ); 
    m_program->link();
    m_posAttr = m_program->attributeLocation("posAttr");
    Q_ASSERT(m_posAttr != -1);
    m_colAttr = m_program->attributeLocation("colAttr");
    Q_ASSERT(m_colAttr != -1);
    m_matrixUniform = m_program->uniformLocation("matrix");
    Q_ASSERT(m_matrixUniform != -1);

	// this->setMouseTracking( false );
}

void TriangleWindow::render()
{
    const qreal retinaScale = devicePixelRatio();
    glViewport(0, 0, width() * retinaScale, height() * retinaScale);

    glClear(GL_COLOR_BUFFER_BIT);
    
    m_program->bind();

    QMatrix4x4 matrix, M;
    M.ortho(-20.0, 20.0, -20.0, 20.0, -20.0, 20.0);
	M.rotate( thetax, 0.0, 1.0, 0.0 );
    
    /////PŔUEBA de multiplicación de las matrices////////////////////////////////////
    // M =  K * RT;
    // float  *d = M.data();
    // for(int i = 0; i<16; i++){
    //     // for(int j = 0; i<16; i++){
    //         printf("%f\n", d[i]);
    //     // }
    // }
    
    // matrix.perspective(60.0f, 4.0f / 3.0f, 0.1f, 100.0f);
    // matrix.translate(0, 0, -2);
    // matrix.rotate(100.0f * m_frame / screen()->refreshRate(), 0, 1, 0);

    m_program->setUniformValue(m_matrixUniform, M);

    // glMultMatrixd(MK);
    // glMultMatrixd(MRT);

    // static const GLfloat vertices[] = {
	
	/** n = 12
	  entonces son 12*2 + 2 = 26
 	  26*3  = 78
	**/

	static const float vertices[78] = {0 , 1.0 , 0.0 ,
		10.0 , 1.0 , 0.0 ,
		0 , 0.8660254037844387 , 0.5,
		10.0 , 0.8660254037844387 , 0.5,
		0 , 0.5000000000000001 , 0.8660254037844386 ,
		10.0 , 0.5000000000000001 , 0.8660254037844386 ,
		0 , 0.0 , 1.0 ,
		10.0 , 0.0 , 1.0 ,
		0 , -0.4999999999999998 , 0.8660254037844387 ,
		10.0 , -0.4999999999999998 , 0.8660254037844387 ,
		0 , -0.8660254037844385 , 0.5,
		10.0 , -0.8660254037844385 , 0.5,
		0 , -1.0 , 0.0,
		10.0 , -1.0 , 0.0,
		0 , -0.8660254037844388 , -0.5,
		10.0 , -0.8660254037844388 , -0.5,
		0 , -0.50000000000000 , -0.8660254037844384 ,
		10.0 , -0.50000000000 , -0.8660254037844384 ,
		0 , -1.8369701987210297e-16 , -1.0 ,
		10.0 , -1.8369701987210297e-16 , -1.0 ,
		0 , 0.5 , -0.866025403784439 ,
		10.0 , 0.5 , -0.866025403784439 ,
		0 , 0.8660254037844384 , -0.5,
		10.0 , 0.8660254037844384 , -0.5,
		0 , 1.0 , 0.0 ,
		10.0 , 1.0 , 0.0 };

	static const float colors[78] = {1.0, 0.0, 0.0,
		1.0, 1.0, 1.0,
		1.0, 0.0, 0.0,
		1.0, 1.0, 1.0,
		1.0, 0.0, 0.0,
		1.0, 1.0, 1.0,
		1.0, 0.0, 0.0,
		1.0, 1.0, 1.0,
		1.0, 0.0, 0.0,
		1.0, 1.0, 1.0, /* 5 */
		1.0, 0.0, 0.0,
		1.0, 1.0, 1.0,
		1.0, 0.0, 0.0,
		1.0, 1.0, 1.0,
		1.0, 0.0, 0.0, 
		1.0, 1.0, 1.0,
		1.0, 0.0, 0.0,
		1.0, 1.0, 1.0,
		1.0, 0.0, 0.0, 
		1.0, 1.0, 1.0, /* 10 */
		1.0, 0.0, 0.0,
		1.0, 1.0, 1.0,
		1.0, 0.0, 0.0,
		1.0, 1.0, 1.0,
		1.0, 0.0, 0.0,
		1.0, 1.0, 1.0 };

    glVertexAttribPointer(m_posAttr, 3, GL_FLOAT, GL_FALSE, 0, vertices);
    glVertexAttribPointer(m_colAttr, 3, GL_FLOAT, GL_FALSE, 0, colors);
   
    glEnableVertexAttribArray(m_posAttr);
    glEnableVertexAttribArray(m_colAttr);

    glDrawArrays(GL_TRIANGLE_STRIP, 0, 26);

    glDisableVertexAttribArray(m_colAttr);
    glDisableVertexAttribArray(m_posAttr);

    m_program->release();
}
//! [5]

void TriangleWindow::inicializa_matrices()
{
    
    ////////////////////////////////////////////////////////////////////

    //     K:
    //  [[  52.1728    0.     -255.    ]
    //  [   0.       52.1728 -255.    ]
    //  [   0.        0.       -1.    ]]
	/**
    MK[0]  = f;    MK[1]  = 0.0; MK[2]  = -255.0; MK[3]  = 0.0;  
    MK[4]  = 0.0;  MK[5]  = f;   MK[6]  = -255.0; MK[7]  = 0.0;  
    MK[8]  = 0.0;  MK[9]  = 0.0; MK[10]  = 101.0; MK[11]  = 100.0;  
    MK[12]  = 0.0; MK[13] = 0.0; MK[14]  = -1.0;  MK[15]  = 0.0;  

    K = QMatrix4x4 (MK);
	*/
    
    // R:
    //  [[ 0.9999  0.0003 -0.0155]
    //  [-0.0001  1.      0.0086]
    //  [ 0.0155 -0.0086  0.9998]]
    // t:
    //  [[ 0.0104]
    //  [ 0.0104]
    //  [-0.5425]]
     
	/**       
    MRT[0 ] = 0.9736;  MRT[1 ] = 0.2282;  MRT[2 ] = -0.0002;  MRT[3 ] =  0.0382 ;    
    MRT[4 ] = -0.0895; MRT[5 ] = 0.3809;  MRT[6 ] =  0.9203;  MRT[7 ] = -0.025; 
    MRT[8 ] = 0.21;    MRT[9 ] = -0.896;  MRT[10] = 0.3913;   MRT[11] = -83.6709;
    MRT[12 ] = 0.0;    MRT[13 ] = 0.0;    MRT[14] = 0.0;      MRT[15] = 1.0;
    RT = QMatrix4x4 (MRT);
    ////////////////////////////////////////

	const float * p = RT.constData();
	for( int i=0; i<16; i++ )
		printf("%g ", p[i] );
	printf("\n" );
	*/
}
